package com.scbfkj.uni.service;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.scbfkj.uni.library.*;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import jakarta.annotation.Resource;
import org.bouncycastle.cert.ocsp.Req;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

import static org.springframework.web.context.request.RequestAttributes.SCOPE_REQUEST;

@Service
public class SecurityService {


    @Value("${app.token-effective:604800}")
    private long defaultAppTokenEffective;
    @Value("${app.code-effective:600}")
    private long defaultSecurityCodeEffective;


    private final static String TYPE = "RSA";

    private final static String ALGORITHM = "RSA/ECB/PKCS1Padding";

    @Value("${cer.private-key}")
    private static String privateKeyStr;
    private final static Map<String, List<String>> alias = new HashMap<>();

    static {
        alias.put("appid", List.of("appid", "app_id", "appId", "APPID"));
        alias.put("appsecret", List.of("appSecret", "app_secret", "APP_SECRET", "appsecret", "APPSECRET"));
        alias.put("sessionid", List.of("sessionId", "sessionid", "SESSIONID", "SESSION_ID", "session_id"));
        alias.put("requestip", List.of("requestIp", "requestip", "request_ip", "REQUEST_IP", "request_ip", "REQUESTIP", "ip"));
        alias.put("username", List.of("username", "userName", "user_name", "USER_NAME", "USERNAME"));
        alias.put("password", List.of("password", "pwd", "PWD", "PASSWORD"));
        alias.put("version", List.of("version", "Version", "VERSION"));
        alias.put("verifycode", List.of("verifycode", "verifyCode", "code"));
    }

    private final static String CLEAN_APPLICATION_CONNECTION_LOG = "delete from appconnectlog where expiretime < NOW() and 1 = ?";
    private final static String QUERY_APPLICATION_BY_APPID_AND_APPSECRET = "select * from application where appid = ? and appsecret";
    private final static String QUERY_APPLICATION_BY_APPID = "select * from application where appid = ? ";
    private final static String QUERY_USER_INFO_BY_ACCOUNT_AND_PASSWORD = "select * from userinfo where  username =? and userpassword=? ";
    private final static String QUERY_USER_INFO_BY_USER_ID = "select * from userinfo where  userid=?";
    private final static String DELETE_USER_LOGIN_LOG_BY_USER_ID = "delete from userloginlog where  userid=?";
    private final static String QUERY_USER_LOGIN_LOG_BY_USER_ID = "select * from userloginlog where userid=?";
    private final static String UPDATE_USER_LOGIN_LOG_WITH_USER_TOKEN_BY_ID = "update userloginlog set apptoken=null,usertoken=?,lasttime=? where  loginid=?";
    private final static String QUERY_USER_LOGIN_LOG_BY_APP_TOKEN_AND_SESSION_ID_AND_IP = "select * from userloginlog where apptoken=? and sessionid=? and requestip=? and isexpires=0 ";
    private final static String INSERT_USER_LOGIN_LOG = "insert into userloginlog ( userid, requestip, sessionid, logintime, usertoken, lasttime, lastheartbeat,apptoken,isexpires,appid)values (?,?,?,?,?,?,?,?,0,?)";
    private final static String QUERY_APP_CONNECT_LOG_BY_APPTOKEN_AND_REQUEST_IP = "select * from appconnectlog where apptoken=? and requestip =?";
    private final static String UPDATE_APP_CONNECT_LOG_WITH_EXPIRE_TIME_BY_APPTOKEN_AND_REQUEST_IP_AND_APPID = "update appconnectlog set expiretime=? where apptoken =? and requestip =? and appid=?";
    private final static String QUERY_USER_LOGIN_LOG_BY_USER_TOKEN_AND_IP = "select * from userloginlog where isexpires=0 and usertoken=? and sessionid=?";

    private final CodeCacheService codeCacheService;

    public SecurityService(CodeCacheService codeCacheService) {

        this.codeCacheService = codeCacheService;
    }

    //安全类服务
    //连接认证--获取连接令牌

    public Map<String, Object> getToken(Map<String, Object> requestData) throws Exception {
        Optional<String> appid = getValue("appid", requestData);
        Optional<String> appSecret = getValue("appsecret", requestData);
        Optional<String> sessionId = getValue("sessionid", requestData);
        if (appSecret.isPresent() && appid.isPresent()) {
            DataBase.updateBatch(Config.securityConnectionStr, CLEAN_APPLICATION_CONNECTION_LOG, new ArrayList<>() {{
                add(new Object[]{1});
            }});
            List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, QUERY_APPLICATION_BY_APPID_AND_APPSECRET, new ArrayList<>() {{
                add(new Object[]{appid.get()});
                add(new Object[]{appSecret.get()});
            }});

            if (applicationList.isEmpty()) {
                return UniReturnUtil.fail("appid 或 appsecret 错误");
            }
            Map<String, Object> application = applicationList.get(0);

            Object apptokeneffectiveObj = application.get("apptokeneffective");
            Long apptokeneffective = defaultAppTokenEffective;
            if (Objects.nonNull(apptokeneffectiveObj)) {
                apptokeneffective = Long.getLong(apptokeneffectiveObj.toString());
            }
            LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(apptokeneffective);
            String appToken = DataEncryptionUtil.signatureMD5("%s:%s".formatted(LocalDateTime.now(), sessionId));
            Map<String, Object> data = new HashMap<>();
            data.put("token", appToken);
            data.put("expirestime", DataFormatUtil.toString(expiresTime));
            data.put("appname", application.get("appname"));
            data.put("appenname", application.get("appengname"));
            data.put("logo", application.get("applog"));
            data.put("smalllogo", application.get("smalllogo"));
            data.put("background", application.get("backgroundimage"));
            data.put("securitycoderule", application.get("securitycoderule"));
            RequestContextHolder.currentRequestAttributes().setAttribute("application", application, SCOPE_REQUEST);
            return UniReturnUtil.success(data);
        }
        return UniReturnUtil.fail("appid 或 appsecret 不能为空");
    }

    //校验连接令牌

    public Map<String, Object> verifyToken(String appToken) throws Exception {

        DataBase.updateBatch(Config.securityConnectionStr, CLEAN_APPLICATION_CONNECTION_LOG, new ArrayList<>() {{
            add(new Object[]{1});
        }});

        String requestIp = RequestUtil.getIpAddr();
        List<Map<String, Object>> appConnectLogList = DataBase.query(Config.securityConnectionStr, QUERY_APP_CONNECT_LOG_BY_APPTOKEN_AND_REQUEST_IP, new ArrayList<>() {{
            add(new Object[]{appToken, requestIp});
        }});
        Map<String, Object> data = new HashMap<>();
        if (!appConnectLogList.isEmpty()) {
            Map<String, Object> appConnectLog = appConnectLogList.get(0);
            data.put("validstatus", true);
            data.put("appid", appConnectLog.get("appid"));
            return UniReturnUtil.success(data);
        }
        data.put("validstatus", false);
        return UniReturnUtil.success(data);
    }

    //刷新连接令牌

    public Map<String, Object> refreshToken() throws Exception {

        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        Object appid = requestAttributes.getAttribute("appid", SCOPE_REQUEST);
        List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, QUERY_APPLICATION_BY_APPID, new ArrayList<>() {{
            add(new Object[]{appid});
        }});
        if (!applicationList.isEmpty()) {
            return UniReturnUtil.fail("应用配置错误");
        }
        Map<String, Object> application = applicationList.get(0);

        Long apptokeneffective = defaultAppTokenEffective;
        Object apptokeneffectiveObj = application.get("apptokeneffective");

        if (Objects.nonNull(apptokeneffectiveObj)) {
            apptokeneffective = Long.getLong(apptokeneffectiveObj.toString());
        }
        LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(apptokeneffective);
        Object appToken = RequestUtil.getAppToken();
        Map<String, Object> data = new HashMap<>();
        try {
            DataBase.updateBatch(Config.securityConnectionStr, UPDATE_APP_CONNECT_LOG_WITH_EXPIRE_TIME_BY_APPTOKEN_AND_REQUEST_IP_AND_APPID, new ArrayList<>() {{
                add(new Object[]{expiresTime, appToken, RequestUtil.getIpAddr(), appid});
            }});
            data.put("expirestime", DataFormatUtil.toString(expiresTime));
            data.put("token", appToken);
            return UniReturnUtil.success(data);
        } catch (Exception exception) {
            return UniReturnUtil.fail("刷新令牌失败：%s".formatted(exception.getMessage()));
        }
    }

    //获取登录验证码

    public Map<String, Object> verifyCode() throws Exception {

        String appId = RequestUtil.getAppId();
        List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, QUERY_APPLICATION_BY_APPID, new ArrayList<>() {{
            add(new Object[]{appId});
        }});
        if (!applicationList.isEmpty()) {
            return UniReturnUtil.fail("应用配置错误");
        }
        Map<String, Object> application = applicationList.get(0);
        Object securityCodeRule = application.get("securitycoderule");
        Object securityCodeEffectiveObj = application.get("securitycodeeffective");
        if (Objects.nonNull(securityCodeRule)) {
            String rule = "";
            String codeRule = (String) securityCodeRule;
//                数字
            if (codeRule.contains("N")) {
                rule += "23456789";
            }
//            大小写字母 world
            if (codeRule.contains("W")) {
                rule += "ABCDEFGHGKMNPQRSTUVWXYZ";
                rule += "abcdefghgkmnpqrstuvwxyz";
//                大写字母 upperCase
            } else if (codeRule.contains("U")) {
                rule += "ABCDEFGHGKMNPQRSTUVWXYZ";
//                小写字母 lower
            } else if (codeRule.contains("L")) {
                rule += "abcdefghgkmnpqrstuvwxyz";
            }
//            字符 c
            if (codeRule.contains("C")) {
                rule += "~!@#%&-_;:";
            }
//            最后四位长度为最小长度和最大长度
            String minMax = codeRule.substring(codeRule.length() - 4);

            Integer min = Integer.getInteger(minMax.substring(0, 2));
            Integer max = Integer.getInteger(minMax.substring(2));

            Integer length = min;
            if (!Objects.equals(min, max)) {
                Random random = new Random();
                do {
                    length = random.nextInt(max);
                } while (length <= min);
            }
            String code = CodeUtil.createCode(length, rule);
            String base64Code = ImageUtil.stringToImage(code);
            String sessionId = RequestUtil.getSessionId();
            String ip = RequestUtil.getIpAddr();
            Long securityCodeEffective = defaultSecurityCodeEffective;
            if (Objects.nonNull(securityCodeEffectiveObj)) {
                securityCodeEffective = Long.getLong(securityCodeEffectiveObj.toString());
            }
            codeCacheService.addCode(code, sessionId, appId, securityCodeEffective, ip);

            Map<String, Object> data = new HashMap<>();
            data.put("verifyCodeImage", base64Code);
            return UniReturnUtil.success(data);
        }
        return UniReturnUtil.fail("没有配置验证码规则");
    }


    //用户登录

    public Map<String, Object> login(Map<String, Object> requestData) throws Exception {

        Optional<String> username = getValue("username", requestData);
        Optional<String> password = getValue("password", requestData);
        Optional<String> verifycode = getValue("verifycode", requestData);
//        解密验证码
        String code = DataEncryptionUtil.decryptByPrivateKey(verifycode.get(), privateKeyStr, TYPE, ALGORITHM);
        String appId = RequestUtil.getAppId();
        List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, QUERY_APPLICATION_BY_APPID, new ArrayList<>() {{
            add(new Object[]{appId});
        }});
        if (!applicationList.isEmpty()) {
            return UniReturnUtil.fail("应用配置错误");
        }
        Map<String, Object> application = applicationList.get(0);
        Object securityCodeRule = application.get("securitycoderule");
        String sessionId = RequestUtil.getSessionId();
        String ip = RequestUtil.getIpAddr();
        if (Objects.nonNull(securityCodeRule) && !codeCacheService.check(code, sessionId, appId, ip)) {
            return UniReturnUtil.fail("验证码错误");
        }

        List<Map<String, Object>> userInfoList = DataBase.query(Config.securityConnectionStr, QUERY_USER_INFO_BY_ACCOUNT_AND_PASSWORD, new ArrayList<>() {{
            add(new Object[]{username.get(), DataEncryptionUtil.decryptByPrivateKey(password.get(), privateKeyStr, TYPE, ALGORITHM)});
        }});
        if (userInfoList.isEmpty()) {
            return UniReturnUtil.fail("用户名密码错误");
        }
        Map<String, Object> userInfo = userInfoList.get(0);
        Object userId = userInfo.get("userid");
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, QUERY_USER_LOGIN_LOG_BY_USER_ID, new ArrayList<>() {{
            add(new Object[]{userInfo.get("userid")});
        }});

        Map<String, Object> data = new HashMap<>();
        String appToken = RequestUtil.getAppToken();
        if (userLoginLogList.isEmpty()) {
            data.put("userstatus", "0");
            DataBase.updateBatch(Config.securityConnectionStr, INSERT_USER_LOGIN_LOG, new ArrayList<>() {{
                add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
            }});
        } else {
            Object multilogin = application.get("multilogin");
            if (Objects.equals(multilogin, "1")) {
                Optional<Map<String, Object>> log = userLoginLogList.stream().filter(it -> it.get("sessionid").equals(sessionId)).findAny();
                if (log.isEmpty()) {
                    DataBase.updateBatch(Config.securityConnectionStr, INSERT_USER_LOGIN_LOG, new ArrayList<>() {{
                        add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
                    }});
                }
                data.put("userstatus", "0");

            } else {
                Optional<Map<String, Object>> log = userLoginLogList.stream().filter(it -> it.get("sessionid").equals(sessionId)).findAny();
                if (log.isEmpty()) {

                    DataBase.updateBatch(Config.securityConnectionStr, INSERT_USER_LOGIN_LOG, new ArrayList<>() {{
                        add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
                    }});
                }
                data.put("userstatus", "1");
            }
        }


        return UniReturnUtil.success(data);
    }


    //强制登录

    public Map<String, Object> forceLogin() throws Exception {
        String appId = RequestUtil.getAppId();
        List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, QUERY_APPLICATION_BY_APPID, new ArrayList<>() {{
            add(new Object[]{appId});
        }});
        if (!applicationList.isEmpty()) {
            return UniReturnUtil.fail("应用配置错误");
        }
        Map<String, Object> application = applicationList.get(0);

        String appToken = RequestUtil.getAppToken();
        String ip = RequestUtil.getIpAddr();
        String sessionId = RequestUtil.getSessionId();
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, QUERY_USER_LOGIN_LOG_BY_APP_TOKEN_AND_SESSION_ID_AND_IP, new ArrayList<>() {{
            add(new Object[]{appToken, sessionId, ip});
        }});
        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("登录失败");
        }
        Map<String, Object> userLoginLog = userLoginLogList.get(0);
        Object securityCodeEffectiveObj = application.get("securitycodeeffective");
        Long securityCodeEffective = defaultSecurityCodeEffective;
        if (Objects.nonNull(securityCodeEffectiveObj)) {
            securityCodeEffective = Long.getLong(securityCodeEffectiveObj.toString());
        }
        LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(securityCodeEffective);
        String userToken = DataEncryptionUtil.signatureMD5("%s:%s".formatted(LocalDateTime.now(), sessionId));
        DataBase.updateBatch(Config.securityConnectionStr, UPDATE_USER_LOGIN_LOG_WITH_USER_TOKEN_BY_ID, new ArrayList<>() {{
            add(new Object[]{userToken, expiresTime, userLoginLog.get("loginid")});
        }});
        HashMap<String, Object> data = new HashMap<>();
        data.put("expirestime", expiresTime);
        data.put("usertoken", userToken);

        return UniReturnUtil.success(data);

    }

    private Map<String, Object> checkUserToken(String userToken) {
        //       Todo
        return null;

    }

    //用户登出

    public Map<String, Object> logOut(Map<String, Object> requestData) throws Exception {
        //       Todo
        String userToken = RequestUtil.getUserToken();
        String sessionId = RequestUtil.getSessionId();
        String ip = RequestUtil.getIpAddr();

        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, QUERY_USER_LOGIN_LOG_BY_USER_TOKEN_AND_IP, new ArrayList<>() {{
            add(new Object[]{userToken, ip});
        }});

        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("登出失败");
        }
        Map<String, Object> userLoginLog = userLoginLogList.get(0);
        Object userIdObj = userLoginLog.get("userid");
        DataBase.updateBatch(Config.securityConnectionStr, DELETE_USER_LOGIN_LOG_BY_USER_ID, new ArrayList<>() {{
            add(new Object[]{userIdObj});
        }});


        return UniReturnUtil.success("成功");
    }


    //获取用户权限

    public Map<String, Object> permission(Map<String, Object> requestData) throws JsonProcessingException {
//       Todo
        return null;
    }

    //应用API及数据权限
    public Map<String, Object> changePassword(Map<String, Object> requestData) throws JsonProcessingException {
        //       Todo
        return null;
    }

    //用户心跳
    public List<Map<String, Object>> userHeartbeat(Map<String, Object> requestData) {
        //       Todo
        return null;
    }


    private Optional<String> getValue(String key, Map<String, Object> data) {
        return alias.getOrDefault(key, Collections.singletonList(key)).stream().map(data::get).filter(Objects::nonNull).map(Object::toString).findAny();
    }
}
